@@ -18,6 +18,7 @@
 
 import os.path
 import unittest
+import time
 
 import dkim
 
@@ -53,12 +54,28 @@ class TestSignAndVerify(unittest.TestCase):
         self.key = read_test_data("test.private")
 
     def dnsfunc(self, domain):
+        sample_dns = """\
+k=rsa; \
+p=MFwwDQYJKoZIhvcNAQEBBQADSwAwSAJBANmBe10IgY+u7h3enWTukkqtUD5PR52T\
+b/mPfjC0QJTocVBq6Za/PlzfV+Py92VaCak19F4WrbVTK5Gg5tW220MCAwEAAQ=="""
+
+        _dns_responses = {
+          'example._domainkey.canonical.com.': sample_dns,
+          'test._domainkey.example.com.': read_test_data("test.txt"),
+          '20120113._domainkey.gmail.com.': """k=rsa; \
+p=MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA1Kd87/UeJjenpabgbFwh\
++eBCsSTrqmwIYYvywlbhbqoo2DymndFkbjOVIPIldNs/m40KF+yzMn1skyoxcTUGCQ\
+s8g3FgD2Ap3ZB5DekAo5wMmk4wimDO+U8QzI3SD07y2+07wlNWwIt8svnxgdxGkVbb\
+hzY8i+RQ9DpSVpPbF7ykQxtKXkv/ahW3KjViiAH+ghvvIhkx4xYSIc9oSwVmAl5Oct\
+MEeWUwg8Istjqz8BZeTWbf41fbNhte7Y+YqZOwq1Sd0DbvYAD9NOZK9vlfuac0598H\
+Y+vtSBczUiKERHv1yRbcaQtZFh5wtiRrN04BLUTD21MycBX5jYchHjPY/wIDAQAB"""
+        }
         try:
             domain = domain.decode('ascii')
         except UnicodeDecodeError:
             return None
-        self.assertEqual('test._domainkey.example.com.', domain)
-        return read_test_data("test.txt")
+        self.assertTrue(domain in _dns_responses,domain)
+        return _dns_responses[domain]
 
     def test_verifies(self):
         # A message verifies after being signed.
@@ -92,14 +109,14 @@ class TestSignAndVerify(unittest.TestCase):
       # <https://bugs.launchpad.net/dkimpy/+bug/939128>
       # Simple-mode signature header verification is wrong
       # (should ignore FWS anywhere in signature tag: b=)
-      sample_msg = """\
+      sample_msg = b"""\
 From: mbp@canonical.com
 To: scottk@example.com
 Subject: this is my
     test message
-""".replace('\n', '\r\n')
+""".replace(b'\n', b'\r\n')
 
-      sample_privkey = """\
+      sample_privkey = b"""\
 -----BEGIN RSA PRIVATE KEY-----
 MIIBOwIBAAJBANmBe10IgY+u7h3enWTukkqtUD5PR52Tb/mPfjC0QJTocVBq6Za/
 PlzfV+Py92VaCak19F4WrbVTK5Gg5tW220MCAwEAAQJAYFUKsD+uMlcFu1D3YNaR
@@ -118,15 +135,9 @@ b/mPfjC0QJTocVBq6Za/PlzfV+Py92VaCak19F4WrbVTK5Gg5tW220MCAwEAAQ==
 -----END PUBLIC KEY-----
 """
 
-      sample_dns = """\
-k=rsa; \
-p=MFwwDQYJKoZIhvcNAQEBBQADSwAwSAJBANmBe10IgY+u7h3enWTukkqtUD5PR52T\
-b/mPfjC0QJTocVBq6Za/PlzfV+Py92VaCak19F4WrbVTK5Gg5tW220MCAwEAAQ=="""
-
-      _dns_responses = {'example._domainkey.canonical.com.': sample_dns}
       for header_mode in [dkim.Relaxed, dkim.Simple]:
 
-        dkim_header = dkim.sign(sample_msg, 'example', 'canonical.com',
+        dkim_header = dkim.sign(sample_msg, b'example', b'canonical.com',
             sample_privkey, canonicalize=(header_mode, dkim.Relaxed))
         # Folding dkim_header affects b= tag only, since dkim.sign folds
         # sig_value with empty b= before hashing, and then appends the
@@ -135,7 +146,7 @@ b/mPfjC0QJTocVBq6Za/PlzfV+Py92VaCak19F4WrbVTK5Gg5tW220MCAwEAAQ=="""
         # simple canonicalization.  
         # http://tools.ietf.org/html/rfc4871#section-3.5
         signed = dkim.fold(dkim_header) + sample_msg
-        result = dkim.verify(signed,dnsfunc=lambda x: _dns_responses[x],
+        result = dkim.verify(signed,dnsfunc=self.dnsfunc,
                 minkey=512)
         self.assertTrue(result)
         dkim_header = dkim.fold(dkim_header)
@@ -143,9 +154,17 @@ b/mPfjC0QJTocVBq6Za/PlzfV+Py92VaCak19F4WrbVTK5Gg5tW220MCAwEAAQ=="""
         pos = dkim_header.rindex(b'\r\n ')
         dkim_header = dkim_header[:pos]+b'\r\n\t'+dkim_header[pos+3:]
         result = dkim.verify(dkim_header + sample_msg,
-                dnsfunc=lambda x: _dns_responses[x], minkey=512)
+                dnsfunc=self.dnsfunc, minkey=512)
         self.assertTrue(result)
 
+    def test_degenerate_folding(self):
+        # <http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=711751>
+        # degenerate folding is ugly but legal
+        message = read_test_data("test2.message")
+        dv = dkim.DKIM(message)
+        res = dv.verify(dnsfunc=self.dnsfunc)
+        self.assertTrue(res)
+    	
     def test_extra_headers(self):
         # <https://bugs.launchpad.net/dkimpy/+bug/737311>
         # extra headers above From caused failure
@@ -155,7 +174,7 @@ b/mPfjC0QJTocVBq6Za/PlzfV+Py92VaCak19F4WrbVTK5Gg5tW220MCAwEAAQ=="""
             for body_algo in (b"simple", b"relaxed"):
                 d = dkim.DKIM(message)
                 # bug requires a repeated header to manifest
-                d.should_not_sign.remove('received')
+                d.should_not_sign.remove(b'received')
                 sig = d.sign(b"test", b"example.com", self.key,
                     include_headers=d.all_sign_headers(),
                     canonicalize=(header_algo, body_algo))
@@ -202,10 +221,41 @@ b/mPfjC0QJTocVBq6Za/PlzfV+Py92VaCak19F4WrbVTK5Gg5tW220MCAwEAAQ=="""
         identity = None
         try:
             sig = dkim.sign(message, selector, domain, read_test_data('test.private'), identity = identity)
-        except dkim.ParameterError as sigerror:
-            pass
+        except dkim.ParameterError as x:
+            sigerror = True
         self.assertTrue(sigerror)
 
+    def test_validate_signature_fields(self):
+      sig = {b'v': b'1',
+      b'a': b'rsa-sha256',
+      b'b': b'K/UUOt8lCtgjp3kSTogqBm9lY1Yax/NwZ+bKm39/WKzo5KYe3L/6RoIA/0oiDX4kO\n \t Qut49HCV6ZUe6dY9V5qWBwLanRs1sCnObaOGMpFfs8tU4TWpDSVXaNZAqn15XVW0WH\n \t EzOzUfVuatpa1kF4voIgSbmZHR1vN3WpRtcTBe/I=',
+      b'bh': b'n0HUwGCP28PkesXBPH82Kboy8LhNFWU9zUISIpAez7M=',
+      b'c': b'simple/simple',
+      b'd': b'kitterman.com',
+      b'i': b'scott@Kitterman.com',
+      b'h': b'From:To:Subject:Date:Cc:MIME-Version:Content-Type:\n \t Content-Transfer-Encoding:Message-Id',
+      b's': b'2007-00',
+      b't': b'1299525798'}
+      dkim.validate_signature_fields(sig)
+      # try new version
+      sigVer = sig.copy()
+      sigVer[b'v'] = 2
+      self.assertRaises(dkim.ValidationError, dkim.validate_signature_fields, sigVer)
+      # try with x
+      sigX = sig.copy()
+      sigX[b'x'] = b'1399525798'
+      dkim.validate_signature_fields(sig)
+      # try with late t
+      sigX[b't'] = b'1400000000'
+      self.assertRaises(dkim.ValidationError, dkim.validate_signature_fields, sigX)
+      # try without t
+      now = int(time.time())
+      sigX[b'x'] = str(now+400000).encode('ascii')
+      dkim.validate_signature_fields(sigX)
+      # try when expired a day ago
+      sigX[b'x'] = str(now - 24*3600).encode('ascii')
+      self.assertRaises(dkim.ValidationError, dkim.validate_signature_fields, sigX)
+
 def test_suite():
     from unittest import TestLoader
     return TestLoader().loadTestsFromName(__name__)
